
import com.google.gson.*;
import com.google.gson.reflect.TypeToken;

import java.lang.reflect.Type;
import java.util.*;

public class JsonHelper {

    public static final Type MAP_TYPE = new TypeToken<Map<String, Object>>() {
    }.getType();

    public static final Type LIST_TYPE = new TypeToken<List<Object>>() {
    }.getType();
    public static final Gson GSON = new GsonBuilder().disableHtmlEscaping()
            .registerTypeAdapter(Double.class, new JsonSerializer<Double>() {
                @Override
                public JsonElement serialize(Double src, Type typeOfSrc, JsonSerializationContext context) {
                    if (src.isNaN()) {
                        return JsonNull.INSTANCE;
                    } else if (src == src.longValue())
                        return new JsonPrimitive(src.longValue());
                    return new JsonPrimitive(src);
                }
            }).registerTypeAdapter(MAP_TYPE, new JsonDeserializer<Map<String, Object>>() {
                @Override
                public Map<String, Object> deserialize(JsonElement json, Type typeOfT, JsonDeserializationContext context) throws JsonParseException {
                    return (Map<String, Object>) read(json);
                }

                private Object read(JsonElement in) {

                    if (in.isJsonArray()) {
                        List<Object> list = new ArrayList<Object>();
                        JsonArray arr = in.getAsJsonArray();
                        for (JsonElement anArr : arr) {
                            list.add(read(anArr));
                        }
                        return list;
                    } else if (in.isJsonObject()) {
                        Map<String, Object> map = new LinkedHashMap<>();//保证属性的顺序
                        JsonObject obj = in.getAsJsonObject();
                        Set<Map.Entry<String, JsonElement>> entitySet = obj.entrySet();
                        for (Map.Entry<String, JsonElement> entry : entitySet) {
                            map.put(entry.getKey(), read(entry.getValue()));
                        }
                        return map;
                    } else if (in.isJsonPrimitive()) {
                        JsonPrimitive prim = in.getAsJsonPrimitive();
                        if (prim.isBoolean()) {
                            return prim.getAsBoolean();
                        } else if (prim.isString()) {
                            return prim.getAsString();
                        } else if (prim.isNumber()) {
                            Number num = prim.getAsNumber();
                            if (Math.ceil(num.doubleValue()) == num.longValue())
                                return num.longValue();
                            else {
                                return num.doubleValue();
                            }
                        }
                    }
                    return null;
                }
            }).create();



    //Json字符串转HashMap
    public static Map<String, Object> jsonDecode(String json) {
        Map<String, Object> returnMap = new HashMap<>();
        try {
            returnMap = GSON.fromJson(json, MAP_TYPE);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return returnMap;
    }

}
