
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.chrome.ChromeDriver;
import org.openqa.selenium.chrome.ChromeOptions;

import java.io.*;
import java.net.HttpURLConnection;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.net.URL;
import java.nio.charset.StandardCharsets;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 假设花漾客户端安装在当前机器，这个程序也跑在这个机器。
 * <p>
 * 下面的Java程序演示在当前电脑上创建打开浏览器，并连接浏览器的remote debug port。
 *
 */
public class SeleniumDemo {
    
    static String accessKeyId = "你的AKID";
    static String accessKeySecret = "你的AK密钥";
    static String endpoint = "https://api.szdamai.com/openapi";

    /**
     * 需要打开的分身ID,必填
     */
    private static final long accountId = 1234567890L;
    /**
     * 需要打开浏览器的设备ID（可选）
     */
    private static String deviceId = null;
    /**
     * 需要浏览器监听的端口
     */
    private static Integer remoteDebugPort = 9222;

    /**
     * 浏览器启动参数。通过换行符\n分割每个参数对。可选
     */
    private static String browserSwitches = null;

    /**
     * chromedriver.exe可以在这里下载，必须是对应花漾的内核版本：
     * https://storage.googleapis.com/chrome-for-testing-public/120.0.6099.109/win64/chromedriver-win64.zip
     */
    private static String chromedriverFile = "D:\\chromedriver.exe";

    public static void main(String[] args) throws IOException {
        System.setProperty("webdriver.chrome.driver", chromedriverFile);
        //获取认证token
        String token = refrshToken();

        //打开会话，获取到异步请求的requestId
        String requestId = openSession(token);

        //等待会话创建完成
        Long sessionId = waitForSessionOpen(token, requestId);
        System.out.println("会话创建完成sessionId=" + sessionId);

        while (true) {//探测远程调试端口是否监听
            try {
                Thread.sleep(2000);
            } catch (InterruptedException e) {
            }
            if (isRemoteReachable("127.0.0.1", remoteDebugPort, 1000)) {
                break;
            }
        }

        //花漾浏览器起来之后会有一个稍长的初始化过程，需要等待一会儿
        try {
            Thread.sleep(10 * 1000);
        } catch (InterruptedException e) {
        }

        ChromeOptions chromeOptions = new ChromeOptions();
        chromeOptions.setExperimentalOption("debuggerAddress", "127.0.0.1:" + remoteDebugPort);
        WebDriver webDriver = new ChromeDriver(chromeOptions);

        webDriver.get("https://www.baidu.com");

        // Get and print the page title
        String pageTitle = webDriver.getTitle();
        System.out.println("Page Title: " + pageTitle);
        try {
            Thread.sleep(3000);
        } catch (InterruptedException e) {
        }
        webDriver.quit();
    }

    private static Long waitForSessionOpen(String token, String requestId) throws IOException {
        Long sessionId = null;
        while (true) {//等待会话创建完成
            try {
                Thread.sleep(2000);
            } catch (InterruptedException e) {
            }
            Map<String, Object> t = findTaskByRequestId(token, requestId);
            if (t != null && Boolean.TRUE.equals(t.get("done"))) {
                if (!"Success".equalsIgnoreCase((String) t.get("status"))) {
                    throw new IOException((String) t.get("remarks"));
                }
                sessionId = (Long) t.get("resourceId");
                System.out.println("当前打开的会话ID=" + sessionId);
                break;
            }
        }
        return sessionId;
    }

    public static String refrshToken() throws IOException {
        Map<String, Object> resp = (Map<String, Object>) request("GET", String.format("/token?accessKeyId=%s&accessKeySecret=%s&expireSeconds=%d", accessKeyId, accessKeySecret, 7200), null, null);
        String token = (String) resp.get("token");
        return token;
    }

    public static String openSession(String token) throws IOException {
        Map<String, String> headers = prepareHeaders(token);
        Map<String, Object> requestBody = new HashMap<>();
        requestBody.put("accountId", accountId);
        if (deviceId != null) {
            requestBody.put("deviceId", deviceId);
        }
        if (remoteDebugPort != null) {
            requestBody.put("remoteDebugPort", remoteDebugPort);
        }
        if (browserSwitches != null) {
            requestBody.put("browserSwitches", browserSwitches);
        }
        Map<String, Object> resp = (Map<String, Object>) request("POST", "/session/openV2", requestBody, headers);
        String requestId = (String) resp.get("requestId");
        return requestId;
    }

    private static Map<String, String> prepareHeaders(String token) {
        Map<String, String> headers = new HashMap<>();
        headers.put("Authorization", token);
        headers.put("Content-Type", "application/json");
        return headers;
    }

    public static Map<String, Object> findTaskByRequestId(String token, String requestId) throws IOException {
        Map<String, String> headers = prepareHeaders(token);
        List resp = (List) request("GET", "/task/" + requestId, null, headers);
        if (resp.size() > 0) {
            return (Map<String, Object>) resp.get(0);
        }
        return Collections.emptyMap();
    }

    public static Object request(String method, String path, Object requestBody, Map<String, String> headers) throws IOException {
        URL url = new URL(endpoint + path);
        HttpURLConnection conn = (HttpURLConnection) url.openConnection();
        conn.setRequestMethod(method);
        if (headers != null) {
            for (Map.Entry<String, String> entry : headers.entrySet()) {
                conn.setRequestProperty(entry.getKey(), entry.getValue());
            }
        }
        if (requestBody != null) {
            conn.setDoOutput(true);
        }
        try {
            conn.connect();
            if (requestBody != null) {
                String json = JsonHelper.GSON.toJson(requestBody);
                writeToStream(json, conn.getOutputStream());
            }
            if (200 <= conn.getResponseCode() && conn.getResponseCode() < 300) {
                String responseJson = copy2String(conn.getInputStream());
                Map<String, Object> resp = JsonHelper.jsonDecode(responseJson);
                if (Boolean.TRUE.equals(resp.get("success"))) {
                    return resp.get("data");
                } else {
                    String msg = (String) resp.get("message");
                    if (msg == null) {
                        msg = conn.getResponseMessage();
                    }
                    throw new IOException("access openapi error :" + msg);
                }
            } else {
                throw new IOException("access openapi error :" + conn.getResponseCode() + " - " + conn.getResponseMessage());
            }
        } finally {
            conn.disconnect();
        }
    }

    public static String copy2String(InputStream inStream) throws IOException {
        ByteArrayOutputStream result = new ByteArrayOutputStream();
        byte[] buffer = new byte[1024];
        int len;
        while ((len = inStream.read(buffer)) != -1) {
            result.write(buffer, 0, len);
        }
        String str = result.toString(StandardCharsets.UTF_8.name());
        return str;
    }

    public static void writeToStream(String str, OutputStream outputStream) {
        BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(outputStream, StandardCharsets.UTF_8));
        try {
            writer.write(str);
        } catch (IOException e) {
            throw new RuntimeException(e);
        } finally {
            try {
                writer.close();
            } catch (IOException e) {
            }
        }
    }

    /**
     * 探测远程端口是否监听
     *
     * @param host
     * @param port
     * @param timeout
     * @return
     */
    public static boolean isRemoteReachable(String host, int port, int timeout) {
        Socket s = null;
        try {
            s = new Socket();
            s.connect(new InetSocketAddress(host, port), timeout);
            return true;
        } catch (Exception e) {
        } finally {
            try {
				s.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
        }
        return false;
    }
}
