/**
 * 要求NodeJs 版本至少为16.20.2
 * 
 * 使用方法:
 * 1、在“团队”=》“团队资源”=>“Open API”页面中查看AccessKey，填入下面的open_api_key和open_api_secret两个变量；
 * 2、选择目标分身点开“编辑分身”按钮，复制“分身ID”，填入下面的to_open_account_id变量；
 * 3、执行npm install axios 安装相关依赖；
 * 4、执行node node-client-demo.js 查看示例效果。
 */


const axios = require('axios');

// 在“团队”=》“团队资源”=>“Open API”页面中查看AccessKey
const open_api_key = '你的AKID';
const open_api_secret = '你的AK密钥';

// 要打开的分身ID，在编辑分身的对话框中查看
const to_open_account_id = 123456;

let access_token = '';
const open_api_endpoint = 'https://api.szdamai.com/api/openapi';

/**
 * 获取 accessToken，之后的所有请求都需要带上这个token。在指定的有效期内这个token可以缓存
 * @return {Promise<*>}
 */
async function getToken() {
    const url = `${open_api_endpoint}/token?accessKeyId=${open_api_key}&accessKeySecret=${open_api_secret}&expireSeconds=300`;
    const response = (await axios.request({
        method: 'GET',
        url: url,
    })).data;
    const data = response.data;
    return data.token;
}

async function waitTask(taskId) {
    for(let i = 0; i < 20; i++) {
        await new Promise((resolve => {
            setTimeout(()=>resolve(1), 1000);
        }));

        const url = `${open_api_endpoint}/task/${taskId}`;
        const response = (await axios.request({
            method: 'GET',
            url: url,
            headers: {
                Authorization: access_token
            }
        })).data;
        let data = response.data[0];
        if(data.status != 'Pending') {
            return data.resourceId;
        }
    }
    throw '打开会话超时';
}

/**
 * 通过 分身名称 打开会话
 * @return {Promise<void>}
 */
async function openSession() {
    const url = `${open_api_endpoint}/session/open`
    const response = (await axios.request({
        method: 'POST',
        url: url,
        headers: {
            Authorization: access_token
        },
        params: {
            accountId: to_open_account_id,
            // deviceId: to_open_device_id
        }
    })).data;
    let sessionId = await waitTask(response.requestId);
    return sessionId;
}

async function closeSession(sessionId) {
    const url = `${open_api_endpoint}/session/${sessionId}/close`
    const response = (await axios.request({
        method: 'PUT',
        url: url,
        headers: {
            Authorization: access_token
        }
    })).data;
    return response;
}

(async function main() {
    try {
        access_token = await getToken();
        console.log(access_token);

        const sessionId = await openSession();
        console.log('打开会话成功,sessionId=' + sessionId);
        
        await new Promise((resolve => {
            setTimeout(()=>resolve(1), 10*1000);
        }));

        console.log('开始关闭会话,sessionId=' + sessionId);
        await closeSession(sessionId);
        console.log('任务完成');
    } catch (e) {
        console.error(e);
    }
})();
