/**
 * 要求NodeJs 版本至少为16.20.2
 * 
 * 使用方法:
 * 1、在“团队”=》“团队资源”=>“Open API”页面中查看AccessKey，填入下面的open_api_key和open_api_secret两个变量；
 * 2、选择目标分身点开“编辑分身”按钮，复制“分身ID”，填入下面的to_open_account_id变量；
 * 3、下载https://storage.googleapis.com/chrome-for-testing-public/120.0.6099.109/win64/chromedriver-win64.zip解压出chromdriver.exe，并修改path_to_webdriver变量的值为chromdriver.exe的路径；
 * 4、执行npm install 安装相关依赖；
 * 5、执行node use-selenium.js 查看示例效果。
 */


const axios = require('axios');
const net = require('net');
const { Builder } = require('selenium-webdriver');
const chrome = require('selenium-webdriver/chrome');

// 在“团队”=》“团队资源”=>“Open API”页面中查看AccessKey
const open_api_key = '你的AKID';
const open_api_secret = '你的AK密钥';

// 要打开的分身ID，在编辑分身的对话框中查看
const to_open_account_id = 123456;

// chromedriver.exe可以在这里下载，必须是对应花漾的内核版本：
// https://storage.googleapis.com/chrome-for-testing-public/120.0.6099.109/win64/chromedriver-win64.zip
const path_to_webdriver = "d:\\chromedriver.exe";

let access_token = '';
const open_api_endpoint = 'https://api.szdamai.com/api/openapi';

/**
 * 获取 accessToken，之后的所有请求都需要带上这个token。在指定的有效期内这个token可以缓存
 * @return {Promise<*>}
 */
async function getToken() {
    const url = `${open_api_endpoint}/token?accessKeyId=${open_api_key}&accessKeySecret=${open_api_secret}&expireSeconds=300`;
    const response = (await axios.request({
        method: 'GET',
        url: url,
    })).data;
    const data = response.data;
    return data.token;
}

async function waitTask(taskId) {
    for(let i = 0; i < 20; i++) {
        await new Promise((resolve => {
            setTimeout(()=>resolve(1), 1000);
        }));

        const url = `${open_api_endpoint}/task/${taskId}`;
        const response = (await axios.request({
            method: 'GET',
            url: url,
            headers: {
                Authorization: access_token
            }
        })).data;
        let data = response.data[0];
        if(data.status != 'Pending') {
            return data.resourceId;
        }
    }
    throw '打开会话超时';
}

async function waitPortOpen(remoteDebugPort) {
    for(let i = 0; i < 20; i++) {
        await new Promise((resolve => {
            setTimeout(()=>resolve(1), 1000);
        }));

        let isOpen = await new Promise(resolve => {
            const client = new net.Socket();
            client.connect(remoteDebugPort, '127.0.0.1', function() {
                client.destroy();
                resolve(true);
            });
            client.on('error', function() {
                resolve(false);
            });
        });

        if(isOpen) {
            return;
        }
    }
    throw '等待端口打开超时';
}

/**
 * 通过 分身名称 打开会话
 * @return {Promise<void>}
 */
async function openSession(remoteDebugPort) {
    const url = `${open_api_endpoint}/session/open`
    const response = (await axios.request({
        method: 'POST',
        url: url,
        headers: {
            Authorization: access_token
        },
        params: {
            accountId: to_open_account_id,
            remoteDebugPort,
            // deviceId: to_open_device_id
        }
    })).data;
    let sessionId = await waitTask(response.requestId);
    return sessionId;
}

async function closeSession(sessionId) {
    const url = `${open_api_endpoint}/session/${sessionId}/close`
    const response = (await axios.request({
        method: 'PUT',
        url: url,
        headers: {
            Authorization: access_token
        }
    })).data;
    return response;
}

async function connectHuaYoung(remoteDebugPort) {
    //等待浏览器ready
    await new Promise((resolve => {
        setTimeout(()=>resolve(1), 5000);
    }));

    const options = new chrome.Options();
    let service = new chrome.ServiceBuilder(path_to_webdriver);
    options.debuggerAddress(`127.0.0.1:${remoteDebugPort}`);
    let driver = await new Builder()
        .forBrowser('chrome')
        .setChromeOptions(options)
        .setChromeService(service)
        .build();

    //拿到 driver 之后就可以对花漾指纹浏览器进行任何操作了
    await driver.get('https://www.qq.com');

    await driver.getTitle().then((title) => {
        console.log('网页标题为:', title);
    })

    //some code ...

    await driver.quit();
}

(async function main() {
    try {
        access_token = await getToken();
        console.log(access_token);

        let remoteDebugPort = 51321; //随便什么端口都可以，后续 selenium 会通过该端口连接花漾指纹浏览器，要确保该端口未被占用
        const sessionId = await openSession(remoteDebugPort);
        console.log('打开会话成功,sessionId=' + sessionId);

        await waitPortOpen(remoteDebugPort);

        await connectHuaYoung(remoteDebugPort);

        console.log('开始关闭会话,sessionId=' + sessionId);
        await closeSession(sessionId);
        console.log('任务完成');
    } catch (e) {
        console.error(e);
    }
})();
