#!/usr/bin/env python3
# -*- coding:utf-8 -*-

'''
 要求Python 版本至少为 3.10.2
 
 使用方法:
 1、在“团队”=》“团队资源”=>“Open API”页面中查看AccessKey；
 2、选择目标分身点开“编辑分身”按钮，查看“分身ID”；
 3、下载https://storage.googleapis.com/chrome-for-testing-public/120.0.6099.109/win64/chromedriver-win64.zip解压出chromdriver.exe，并将修改path_to_webdriver变量的值为chromdriver.exe的路径；
 4、执行 pip install -r requirements.txt 安装相关依赖；
 5、执行 python SeleniumSetDownloadDir.py {AccessKey ID}  {AccessKey Secret}  {分身ID} 查看示例效果。
'''
from time import sleep
from selenium import webdriver
from selenium.webdriver.chrome.service import Service
from selenium.webdriver.chrome.options import Options

import sys
import requests
import json

# chromedriver.exe可以在这里下载，必须是对应花漾的内核版本：
# https://storage.googleapis.com/chrome-for-testing-public/120.0.6099.109/win64/chromedriver-win64.zip
path_to_webdriver = 'd:/chromedriver.exe'

apiEndpoint = "https://api.szdamai.com/openapi"
'''
openApi: function to access api url and post parameters
'''

def openApi(url: str, params: dict = None, headers: dict = None, method: str = "GET", contentType: str = "json", timeout: int = 10) -> dict:
    if method == "GET":
        response = requests.get(
            apiEndpoint + url, params=params, headers=headers, timeout=timeout)
    elif method == "POST":
        if not headers:
            headers = dict()
        if contentType == "json":
            headers["Content-Type"] = "application/json"
            params = json.dumps(params) if params else None
        response = requests.post(
            apiEndpoint + url, data=params, headers=headers, timeout=timeout)
    elif method == "DELETE":
        response = requests.delete(
            apiEndpoint + url, data=params, headers=headers, timeout=timeout)
    else:
        return None
    if not response:
        return None
    if not 200 <= response.status_code <= 299:
        return None
    result = json.loads(response.text)
    if result["success"]:
        return result["data"]
    else:
        print("openApi %s error %s" % (url, result["message"]))
        return None


def getToken(accessKeyId, accessKeySecret, expireSeconds=300):
    tokenJson = openApi(
        "/token",
        params={
            "accessKeyId": accessKeyId,
            "accessKeySecret": accessKeySecret,
            "expireSeconds": expireSeconds
        },
    )
    return tokenJson["token"]


def getCurrentUser(token):
    hostJson = openApi(
        "/user/current",
        params={},
        headers={"Authorization": token}
    )
    return hostJson


def openSession(token, accountId):
    browserSwitches = "--window-size=700,600\n--window-position=100,50"
    hostJson = openApi(
        "/session/openV2",
        params={
            "accountId": accountId,
            "remoteDebugPort": 9222,
            "browserSwitches": browserSwitches,
        },
        method="POST",
        headers={"Authorization": token}
    )
    return hostJson


def useSelenium():
    chrome_options = Options()
    chrome_options.add_experimental_option("debuggerAddress", "127.0.0.1:9222")

    service = Service(path_to_webdriver)
    driver = webdriver.Chrome(service=service, options=chrome_options)

    # 设置下载路径
    download_path = "d:\\11\\"
    driver.command_executor._commands["send_command"] = ("POST", '/session/$sessionId/chromium/send_command')
    params = {
        'cmd': 'Page.setDownloadBehavior',
        'params': {
            'behavior': 'allow',
            'downloadPath': download_path
        }
    }
    command_result = driver.execute("send_command", params)

    # 打开网站
    driver.get('http://sahitest.com/demo/saveAs.htm')
    driver.find_element("xpath", '//a[text()="testsaveas.zip"]').click()

    sleep(10)
    print(driver.title)

    driver.quit()


def main(args, **kwargs):
    accessKeyId = args[0]
    accessKeySecret = args[1]
    accountId = args[2]
    if not accessKeyId or not accessKeySecret:
        print("AccessKeyId and AccessKeySecret are not present.\n")
        return -1
    token = getToken(accessKeyId, accessKeySecret)
    if not token:
        print("Invalid AccessKeyId and AccessKeySecret.\n")
        return -1
    user = getCurrentUser(token=token)
    if user:
        print("Current user is  %s.\n" % (user))
        rs = openSession(token, accountId)
        print(rs)
        sleep(10)
        useSelenium()
    else:
        print("get Current user fail\n")


if __name__ == "__main__":
    if len(sys.argv) != 4:
        print("Usage: python SeleniumTest.py AccessKeyId AccessKeySecret accountId\n")
        sys.exit(-1)
    sys.exit(main(sys.argv[1:4], **dict(arg.split('=')
             for arg in sys.argv[5:])))