from time import sleep
from selenium import webdriver
from selenium.webdriver.chrome.options import Options

#!/usr/bin/env python3
# -*- coding:utf-8 -*-
import sys
import requests
import json
apiEndpoint = "https://api.szdamai.com/openapi"
'''
openApi: function to access api url and post parameters
'''


def openApi(url: str, params: dict = None, headers: dict = None, method: str = "GET", contentType: str = "json", timeout: int = 10) -> dict:
    if method == "GET":
        response = requests.get(
            apiEndpoint + url, params=params, headers=headers, timeout=timeout)
    elif method == "POST":
        if not headers:
            headers = dict()
        if contentType == "json":
            headers["Content-Type"] = "application/json"
            params = json.dumps(params) if params else None
        response = requests.post(
            apiEndpoint + url, data=params, headers=headers, timeout=timeout)
    elif method == "DELETE":
        response = requests.delete(
            apiEndpoint + url, data=params, headers=headers, timeout=timeout)
    else:
        return None
    if not response:
        return None
    if not 200 <= response.status_code <= 299:
        return None
    result = json.loads(response.text)
    if result["success"]:
        return result["data"]
    else:
        print("openApi %s error %s" % (url, result["message"]))
        return None


def getToken(accessKeyId, accessKeySecret, expireSeconds=300):
    tokenJson = openApi(
        "/token",
        params={
            "accessKeyId": accessKeyId,
            "accessKeySecret": accessKeySecret,
            "expireSeconds": expireSeconds
        },
    )
    return tokenJson["token"]


def getCurrentUser(token):
    hostJson = openApi(
        "/user/current",
        params={},
        headers={"Authorization": token}
    )
    return hostJson


def starTask(token, taskName, deviceId, flowName, shopList, params):
    flowJson = openApi(
        "/rpa/flows?name="+flowName,
        method="GET",
        headers={"Authorization": token}
    )
    if flowJson and flowJson["list"] and len(["list"]) > 0:
        rpaFlowId = flowJson["list"][0]["id"]
        paramsMap = {}
        for shopId in shopList:
            paramsMap[shopId] = params

        postData = {
            "concurrent": 0,
            "concurrentDelay": 0,
            "deviceId": deviceId,  # 在“偏好设置”-》“登录设备”中找
            "forceRecord": False,  # 是否录像
            "name": taskName,  # 设置任务名称
            "rpaFlowId": rpaFlowId,  # 流程ID，通过调用/api/openapi/rpa/flows获取
            "runOnCloud": False,
            "shopIds": shopList,  # 分身ID，在分身属性中查看
            "params": paramsMap,
            "showMouseTrack": False,
            "snapshot": "OnFail",
            "provider": "aliyun",
            "region": "cn-hongkong",
            "runOnCloud": False,
        }
        print(postData)
        rpaJson = openApi(
            "/rpa/triggerTask",
            params=postData,
            method="POST",
            headers={"Authorization": token}
        )
        return rpaJson

    return "无法找到此流程定义"


def main(args, **kwargs):
    accessKeyId = args[0]
    accessKeySecret = args[1]
    deviceId = args[2]

    flowName = "打开百度"
    taskName = "打开百度任务1"
    shopList = [111, 222]
    params = {
        "xxx1": "cdddffd",
        "xxx2": 3
    }

    if not accessKeyId or not accessKeySecret:
        print("AccessKeyId and AccessKeySecret are not present.\n")
        return -1
    token = getToken(accessKeyId, accessKeySecret)
    if not token:
        print("Invalid AccessKeyId and AccessKeySecret.\n")
        return -1
    user = getCurrentUser(token=token)
    if user:
        print("Current user is  %s.\n" % (user))
        rs = starTask(token, taskName, deviceId, flowName, shopList, params)
        print(rs)
        sleep(10)
    else:
        print("get Current user fail\n")


if __name__ == "__main__":
    if len(sys.argv) != 4:
        print("Usage: python python_rpa_demo.py AccessKeyId AccessKeySecret deviceId\n")
        sys.exit(-1)
    sys.exit(main(sys.argv[1:4], **dict(arg.split('=')
             for arg in sys.argv[5:])))
